import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import jp.hozo.core.Concept;
import jp.hozo.core.RoleConcept;
import jp.hozo.core.RoleHolder;
import jp.hozo.core.Slot;
import jp.hozo.core.WholeConcept;
import jp.hozo.corex.HZOntology;

import org.jdesktop.swingx.treetable.AbstractTreeTableModel;

public class ConceptTreeTableModel extends AbstractTreeTableModel {
	public static final String VALUE_DELIMITER = ";";
	public static final String E_A_DELIMITER = ":";
	public static final String BLANK_CHAR = "-";
	public static final String MERAGED_LABEL = ":[merged]";

	// ̑IuWFNg
	protected HZOntology ont;

	// [gi\Ȃj
	protected ConceptTreeNode root;
	
    // Names of the columns.
    protected String[]  columnNames;
    
    // Names of the columns.
    private List<String>  columnNameList;
    
    // rootConcept
    private Concept rootConcept;
    
//    // Cell valuei[Xg
//    private List<Object> cellValueList = new ArrayList<Object>();
    
    // Cell valuei[}bv
    private Map<Concept, List<String>> cellValueListMap = new LinkedHashMap<Concept, List<String>>();
    private Map<String, String> cellValueMap = new LinkedHashMap<String, String>();
    
    // EAV̊i[}bv
    private Map<String, List<Concept>> eavConceptMap = new LinkedHashMap<String, List<Concept>>();
    
    protected List<ConceptTreeNode> nodeList = new ArrayList<ConceptTreeNode>();
    
    private Concept attributeConcept;

    private Concept valueConcept;
    
    private Concept eavConcept;
    
    // sƗւ邩ǂH
    private boolean isReplacedLineToColumn = false;

	// J폜邩ǂH
	private boolean isRemovedEmptyColumn = false;

	// l}XN邩ǂH
    private boolean isValueMasked = false;
    private static final String MASK_SYMBOL = "";

	public ConceptTreeTableModel(Concept entity, HZOntology ont, String[] colNames, Concept attributeConcept, Concept valueConcept, Concept eavCon)	{
		this(entity, ont, colNames, attributeConcept, valueConcept, eavCon, false);
	}
	public ConceptTreeTableModel(Concept entity, HZOntology ont, String[] colNames,  Concept attributeConcept, Concept valueConcept, Concept eavCon, boolean isMasked)	{
		this(entity, ont, colNames, attributeConcept, valueConcept, eavCon, isMasked, false, false);
	}
	public ConceptTreeTableModel(Concept entity, HZOntology ont, String[] colNames,  Concept attributeConcept, Concept valueConcept, Concept eavCon, 
									boolean isMasked, boolean isReplacedLineToColumn, boolean isRemovedEmptyColumn)	{
		this.ont = ont;
		this.rootConcept = entity;
		this.attributeConcept = attributeConcept;
		this.valueConcept = valueConcept;
		this.eavConcept = eavCon;
		this.setColumnNames(colNames);
		this.root = new ConceptTreeNode( entity, new ArrayList<String>() );
		this.isValueMasked = isMasked;
		this.isReplacedLineToColumn = isReplacedLineToColumn;
		this.isRemovedEmptyColumn = isRemovedEmptyColumn;

		ConceptTreeNode entityNode = new ConceptTreeNode(entity, getSubNodeValueList(entity));
		this.nodeList.add(entityNode);
//		List<ConceptTreeNode> subNodes = getSubNodeList(entity);
//		for (ConceptTreeNode subNode : subNodes) {
//			entityNode.getChildren().add(subNode);
//		}
		appendSubNodeList(entityNode, entity);
		//this.root.getChildren().add(entityNode);
		this.root.addChild(entityNode);
	}
	
    
	/**
	 * sƗւ邩ǂH
	 * 
	 * @return isReplacedLineToColumn
	 */
	public boolean isReplacedLineToColumn() {
		return isReplacedLineToColumn;
	}
	
    /**
     *  J폜邩ǂH
     * 
	 * @return isRemovedEmptyColumn
	 */
	public boolean isRemovedEmptyColumn() {
		return isRemovedEmptyColumn;
	}
	
	@Override
    public int getColumnCount() {
    	return columnNames.length;
    }
	
	@Override
    public String getColumnName(int column) {
		if (isRemovedEmptyColumn) {
			return getColumnNameList().get(column);
		}
    	return columnNames[column];
    }

	@Override
	public Object getValueAt( Object node, int column ) {
		ConceptTreeNode treenode;
		if (node instanceof Integer) {
			treenode = nodeList.get(((Integer) node).intValue());
		} else {
			treenode = (ConceptTreeNode) node;
		}
		if (treenode != null) {
			// Ic[̃xl擾
			Concept selectCon = treenode.getConcept();
			List<String> valueList = this.cellValueListMap.get(selectCon);
			if (column == 0) {
				return selectCon.getLabel();
			} else if (valueList != null && !valueList.isEmpty()) {
				return valueList.get(column);
			} else {
				return "?";
			}
		} else {
			return "";
		}
	}

	@Override
	public Object getChild( Object node, int index ) 
	{
		ConceptTreeNode treenode = ( ConceptTreeNode )node;
		return treenode.getChildren().get( index );
	}

	@Override
	public int getChildCount( Object parent ) 
	{
		ConceptTreeNode treenode = ( ConceptTreeNode )parent;
		return treenode.getChildren().size();
	}

	@Override
	public int getIndexOfChild( Object parent, Object child ) 
	{
		ConceptTreeNode treenode = ( ConceptTreeNode )parent;
		for( int i=0; i>treenode.getChildren().size(); i++ )
		{
			if( treenode.getChildren().get( i ) == child )
			{
				return i;
			}
		}

		return 0;
	}
	
	public boolean isLeaf( Object node )
	{
		ConceptTreeNode treenode = ( ConceptTreeNode )node;
		if( treenode.getChildren().size() > 0 )
		{
			return false;
		}
		return true;
	}
	
	@Override
	public Object getRoot()
	{
		return root;
	}

	public String[] getColumnNames() {
		return columnNames;
	}

	public void setColumnNames(String[] cNames) {
		this.columnNames = cNames;
		
		columnNameList = new ArrayList<String>();
		for (int i = 0; i < cNames.length; i++) {
			columnNameList.add(cNames[i]);
		}
	}
	
	public List<String> getColumnNameList() {
		return columnNameList;
	}
	
	@Override
	public boolean isCellEditable(Object node, int column) {
		return super.isCellEditable(node, column);
	}

	/**
	 * rootm[h̉ʊTOXg擾
	 * @param concept
	 * @return
	 */
	private List<ConceptTreeNode> appendSubNodeList(ConceptTreeNode node, Concept concept) {
		List<ConceptTreeNode> rtnList = new ArrayList<ConceptTreeNode>();

		List<Concept> concepts = new ArrayList<Concept>(concept.getLowerConceptList());
		if (concept.hasRoleHolders()) {
			concepts.addAll(concept.getRoleHolderList());
		}
		// IgW[̊TOXg琶
		for (Concept cp : concepts) {
			if (cp != null) {
				// ʊTOꍇASĐVm[hŒǉ
//				if (cp.hasLowerConcept()) {
				if (cp.hasLowerConcept() || cp.hasRoleHolders()) {
					ConceptTreeNode subTreeNode = new ConceptTreeNode(cp, getSubNodeValueList(cp));
					//node.getChildren().add(subTreeNode);
					node.addChild(subTreeNode);
					rtnList.add(subTreeNode);
					this.nodeList.add(subTreeNode);
					appendSubNodeList(subTreeNode, cp);	// qm[hǉ
				} else {
					ConceptTreeNode subTreeNode = new ConceptTreeNode(cp, getSubNodeValueList(cp));
					//node.getChildren().add(subTreeNode);
					node.addChild(subTreeNode);
					rtnList.add(subTreeNode);
					this.nodeList.add(subTreeNode);
				}
			}
		}
	
		return rtnList;
	}
	
	/**
	 * m[hXgTÕx擾
	 */
	protected List<String> getSubNodeValueList(Concept concept) {
		List<String> rtnList = new ArrayList<String>();
		// YsValueSċɐݒ
		for (int i = 0; i < columnNames.length; i++) {
			rtnList.add(BLANK_CHAR);
		}	
		
		// EAVValue擾
		Concept eavCon = this.eavConcept;
		for (Concept tmpEavCon : eavCon.getLowerConceptList()) {
			boolean hasE = false;
			//List<Slot> entitySlots = tmpEavCon.findSlotsByRoleName("E");
			List<Slot> entitySlots;
			if (isReplacedLineToColumn) {
				entitySlots = tmpEavCon.findSlotsByRoleName("A");
			} else {
				entitySlots = tmpEavCon.findSlotsByRoleName("E");
			}
			for (Slot entySlot : entitySlots) {
				if (entySlot != null && entySlot.getConstraintConcept() != null
						&& (entySlot.getConstraintConcept().equals(concept)
								|| entySlot.getConstraintConceptLabel().equals(concept.getLabel()))) {
					hasE = true;
					break;
				}
			}
			Concept attrCon = null;
//			List<Slot> attributeSlots = tmpEavCon.findSlotsByRoleName("A");
			List<Slot> attributeSlots;
			if (isReplacedLineToColumn) {
				attributeSlots = tmpEavCon.findSlotsByRoleName("E");
			} else {
				attributeSlots = tmpEavCon.findSlotsByRoleName("A");
			}
			for (Slot attrSlot : attributeSlots) {
				if (attrSlot != null && attrSlot.getConstraintConcept() != null) {
					attrCon = attrSlot.getConstraintConcept();
					break;
				}
			}
			Concept valueCon = null;
			List<Slot> valueSlots = tmpEavCon.findSlotsByRoleName("V");
			for (Slot valSlot : valueSlots) {
				if (valSlot != null && valSlot.getConstraintConcept() != null) {
					valueCon = valSlot.getConstraintConcept();
					break;
				}
			}
			if (hasE && attrCon != null && valueCon != null 
					&& valueCon.isAncestorConcept(valueConcept)) {		// l̊TOŏo͂𐧖񂷂
				String key;
				key = concept.getLabel() + E_A_DELIMITER + attrCon.getLabel();
				
				List<Concept> eavCons = eavConceptMap.get(key);
				
				int i = 0;
				for (String colName : columnNames) {
					if (colName.equals(attrCon.getLabel())) {
						String curValue = rtnList.get(i);
						if (curValue == BLANK_CHAR) {
							curValue = valueCon.getLabel();
						} else {
							curValue = curValue + VALUE_DELIMITER + valueCon.getLabel();
						}
						
						// Xgɒǉ
						if (eavCons == null) eavCons = new ArrayList<Concept>();
						eavCons.add(tmpEavCon);
						// }bvɂǉ
						cellValueMap.put(key, curValue);
						
						rtnList.remove(i);	// UAēւ
						if (isValueMasked) {
							rtnList.add(i, MASK_SYMBOL);
						} else {
							rtnList.add(i, curValue);
						}
					}
					i++;
				}
				eavConceptMap.put(key, eavCons);
			}
			
//			cellValueList.add(rtnList);
			cellValueListMap.put(concept, rtnList);
		} 
		
		return rtnList;
	}
	
	public Concept getRootConcept() {
		return rootConcept;
	}

	public void setRootConcept(Concept rootConcept) {
		this.rootConcept = rootConcept;
	}

	public List<ConceptTreeNode> getNodeList() {
		return nodeList;
	}

	public void setNodeList(List<ConceptTreeNode> nodeList) {
		this.nodeList = nodeList;
	}
	
	public Map<String, List<Concept>> getEAVConceptMap() {
		return eavConceptMap;
	}
	
	public List<Concept> getEAVConceptList(String key) {
		return getEAVConceptMap().get(key);
	}
	
	public Map<String, String> getCellValueMap() {
		return cellValueMap;
	}
	
	public Map<Concept, List<String>> getCellValueListMap() {
		return cellValueListMap;
	}
	
	public String getCellValue(String key) {
		//System.out.println("getCellValue(" + key + ") = [" + getCellValueMap().get(key) + "]");
		return getCellValueMap().get(key);
	}
	
	public void putCellValue(String key, String value) {
		//System.out.println("putCellValue(" + key + ", " + value + ")");
		getCellValueMap().put(key, value);
	}
	
	
	/**
	 * 
	 * l}XN邩ǂ𔻕ʂ
	 * 
	 * @return isValueMasked
	 */
	public boolean isValueMasked() {
		return isValueMasked;
	}

	/**
	 * 
	 * l}XN邩ǂݒ肷
	 * 
	 * @param isValueMasked l}XN邩ǂw
	 */
	public void setValueMasked(boolean isValueMasked) {
		this.isValueMasked = isValueMasked;
	}
}
