
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import jp.hozo.core.Concept;
import jp.hozo.core.RoleHolder;
import jp.hozo.core.Slot;
import jp.hozo.corex.HZOntology;

public class MetadataTableModel extends ConceptTreeTableModel {
	
    // Cell valuei[}bv
    private Map<Object, List<String>> slotValueListMap = new LinkedHashMap<Object, List<String>>();

	public MetadataTableModel(Concept entityConcept, HZOntology ont, String[] colNames, Concept attributeConcept, Concept valueConcept, Concept eavCon)	{
		this(entityConcept, ont, colNames, attributeConcept, valueConcept, eavCon, false);
	}
	public MetadataTableModel(Concept entityConcept, HZOntology ont, String[] colNames, Concept attributeConcept, Concept valueConcept, Concept eavCon, boolean isMasked)	{
		this(entityConcept, ont, colNames, attributeConcept, valueConcept, eavCon, isMasked, false, false);
	}
	public MetadataTableModel(Concept entityConcept, HZOntology ont, String[] colNames, Concept attributeConcept, Concept valueConcept, Concept eavCon,
								boolean isMasked, boolean isReplacedLineToColumn, boolean isRemovedEmptyColumn)	{
		super(entityConcept, ont, colNames, attributeConcept, valueConcept, eavCon, isMasked, isReplacedLineToColumn, isRemovedEmptyColumn);
		
		ConceptTreeNode attibuteNode = new ConceptTreeNode(attributeConcept, getEmptyValueList());
		super.nodeList.add(attibuteNode);
		this.slotValueListMap.put(attributeConcept, attibuteNode.getValueList());
		appendSubNodeList(attibuteNode, attributeConcept);
		root.getChildren().clear();	// UNA
		//root.getChildren().add(attibuteNode);
		root.addChild(attibuteNode);
	}
	
	/**
	 * m[hXgTÕx擾
	 */
	protected List<String> getEmptyValueList() {
		List<String> rtnList = new ArrayList<String>();
		// YsValueSċɐݒ
		for (int i = 0; i < columnNames.length; i++) {
			rtnList.add(BLANK_CHAR);
		}
		return rtnList;
	}
	
	/**
	 * rootm[h̉ʊTOXg擾
	 * @param concept
	 * @return
	 */
	private List<ConceptTreeNode> appendSubNodeList(ConceptTreeNode node, Concept concept) {
		List<ConceptTreeNode> rtnList = new ArrayList<ConceptTreeNode>();

		//List<Slot> slots = concept.getSlotList();
		List<Slot> slots = getAllSlotListInAllSubConcepts(concept);
		// TÕXbgXg琶
		for (Slot slot : slots) {
			ConceptTreeNode subTreeNode = new SlotTreeNode(slot, getEmptyValueList());
			//node.getChildren().add(subTreeNode);
			node.addChild(subTreeNode);
			rtnList.add(subTreeNode);
			this.nodeList.add(subTreeNode);
			String key = slot.getRoleName();
			this.slotValueListMap.put(key, subTreeNode.getValueList());
			//appendSubNodeList(subTreeNode, slot);	// qm[hǉ
		}
		return rtnList;
	}
	private List<ConceptTreeNode> appendSubNodeList(ConceptTreeNode node, Slot slot) {
		List<ConceptTreeNode> rtnList = new ArrayList<ConceptTreeNode>();

		List<Slot> slots = slot.getSlotList();
		// TÕXbgXg琶
		for (Slot chiSlot : slots) {
			ConceptTreeNode subTreeNode = new SlotTreeNode(chiSlot, getEmptyValueList());
			//node.getChildren().add(subTreeNode);
			node.addChild(subTreeNode);
			rtnList.add(subTreeNode);
			this.nodeList.add(subTreeNode);
			String key = chiSlot.getRoleName();
			this.slotValueListMap.put(key, subTreeNode.getValueList());
			appendSubNodeList(subTreeNode, chiSlot);	// qm[hǉ
		}
		return rtnList;
	}
	
	// ʊTÕXbgׂĎ擾
	private List<Slot> getAllSlotListInAllSubConcepts(Concept concept) {
		List<Slot> rtnList = new ArrayList<Slot>();
		// ΏۊTÕXbg܂߂
		List<Slot> slots = concept.getSlotList();
		if (!slots.isEmpty()) {
			rtnList.addAll(slots);
		}
		List<Concept> subConcepts = new ArrayList<Concept>(concept.getLowerConcepts());
		if (concept.hasRoleHolders()) {
			subConcepts.addAll(concept.getRoleHolderList());
		}
		for (Concept subConcept : subConcepts) {
			List<Slot> subSlots = subConcept.getSlotList();
			if (!slots.isEmpty()) {
				for (Slot subSlot : subSlots) {
					if (!containsSuperSlot(subSlot, rtnList)) {
						rtnList.add(subSlot);
					}
				}
			}
			// ʊTOׂ
			List<Slot> subSlotsInSubConcept = getAllSlotListInAllSubConcepts(subConcept);
			if (!subSlotsInSubConcept.isEmpty()) {
				for (Slot subSubSlot : subSlotsInSubConcept) {
					if (!containsSuperSlot(subSubSlot, rtnList)) {
						rtnList.add(subSubSlot);
					}
				}
			}
		}
		return rtnList;
	}
	
	// ʃXbg܂ł邩ǂ𔻕ʂ
	private boolean containsSuperSlot(Slot slot, List<Slot> slotList) {
		for (Slot tmpSlot : slotList) {
			if (slot.isAncestorSlot(tmpSlot)	
					|| slot.getRoleName().equals(tmpSlot.getRoleName())) {	// ʃXbg[vĂ
				return true;
			}
		}
		return false;
	}
	
	@Override
	public Object getValueAt( Object node, int column ) {
		ConceptTreeNode treenode;
		if (node instanceof Integer) {
			treenode = nodeList.get(((Integer) node).intValue());
		} else {
			treenode = (ConceptTreeNode) node;
		}
		if (treenode != null) {
			// Ic[̃xl擾
			Concept selectCon = treenode.getConcept();
			if (column == 0) {
				return selectCon.getLabel();
			} else if (treenode instanceof SlotTreeNode) {
				Slot slot = ((SlotTreeNode) treenode).getSlot();
				List<String> valueList = slotValueListMap.get(slot.getRoleName());
				if (valueList != null) {
					return valueList.get(column);
				} else {
					return ConceptTreeTableModel.BLANK_CHAR;
				}
			} else if (treenode instanceof ConceptTreeNode) {
				List<String> valueList = slotValueListMap.get(selectCon);
				if (valueList != null) {
					return valueList.get(column);
				} else {
					return ConceptTreeTableModel.BLANK_CHAR;
				}
			} else {
				return "?";
			}
		} else {
			return "";
		}
	}
	
	public void setMetadataValue() {
		int i = 0;
		for (String colName : columnNames) {
			Concept colConcept = ont.findWholeConceptByLabel(colName);
			if (colConcept == null) {
				colConcept = ont.findConceptByLabel(colName, RoleHolder.CONCEPT_TYPE);
			}
			if (colConcept != null) {
				for (Slot slot : colConcept.getSlotList()) {
					String key = slot.getRoleName();
					List<String> valueList = slotValueListMap.get(key);
					if (valueList != null) {
						valueList.set(i, slot.getConstraintConceptLabel());
					}
				}
			}
			i++;
		}
	}

	/**
	 * @return slotValueListMap
	 */
	public Map<Object, List<String>> getSlotValueListMap() {
		return slotValueListMap;
	}
}
